@@ -3,6 +3,7 @@
 mod db;
 mod error;
 mod genre;
+mod progress;
 mod ratings;
 mod service;
 mod title;
@@ -11,7 +12,7 @@ mod title_id;
 mod title_type;
 mod utils;
 
-pub use db::QueryType as ImdbQueryType;
+pub use db::Query as ImdbQuery;
 pub use error::Err as ImdbErr;
 pub use genre::{Genre as ImdbGenre, Genres as ImdbGenres};
 pub use service::Service as Imdb;
